@@ -197,6 +197,7 @@ macro_rules! sequence {
                     out.$insert(T::decode(value)?);
                 }
 
+                $access.end()?;
                 Ok(out)
             }
         }
@@ -252,6 +253,7 @@ macro_rules! map {
                     let mut entry = map.next()?;
                     k.encode(entry.first()?)?;
                     v.encode(entry.second()?)?;
+                    entry.end()?;
                 }
 
                 map.end()
@@ -279,6 +281,7 @@ macro_rules! map {
                     out.insert(key, value);
                 }
 
+                $access.end()?;
                 Ok(out)
             }
         }
