@@ -3,8 +3,11 @@
 
 use core::{fmt, marker};
 
-use crate::de::{Decoder, PackDecoder, PairDecoder, PairsDecoder, SequenceDecoder, VariantDecoder};
+use crate::de::{
+    AsDecoder, Decoder, PackDecoder, PairDecoder, PairsDecoder, SequenceDecoder, VariantDecoder,
+};
 use crate::en::{Encoder, PairEncoder, PairsEncoder, SequenceEncoder, VariantEncoder};
+use crate::error::Error;
 
 enum NeverMarker {}
 
@@ -24,14 +27,14 @@ enum NeverMarker {}
 ///
 /// impl Decoder<'_> for MyDecoder {
 ///     type Error = String;
-///     type Pack = Never<Self>;
-///     type Sequence = Never<Self>;
-///     type Tuple = Never<Self>;
-///     type Map = Never<Self>;
-///     type Some = Never<Self>;
-///     type Struct = Never<Self>;
-///     type TupleStruct = Never<Self>;
-///     type Variant = Never<Self>;
+///     type Buffer = Never<Self::Error>;
+///     type Pack = Never<Self::Error>;
+///     type Sequence = Never<Self::Error>;
+///     type Tuple = Never<Self::Error>;
+///     type Map = Never<Self::Error>;
+///     type Some = Never<Self::Error>;
+///     type Struct = Never<Self::Error>;
+///     type Variant = Never<Self::Error>;
 ///
 ///     fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
 ///         write!(f, "32-bit unsigned integers")
@@ -48,18 +51,18 @@ pub struct Never<T> {
     _marker: marker::PhantomData<T>,
 }
 
-impl<'de, T> Decoder<'de> for Never<T>
+impl<'de, E> Decoder<'de> for Never<E>
 where
-    T: Decoder<'de>,
+    E: Error,
 {
-    type Error = T::Error;
+    type Error = E;
+    type Buffer = Self;
     type Pack = Self;
     type Sequence = Self;
     type Tuple = Self;
     type Map = Self;
     type Some = Self;
     type Struct = Self;
-    type TupleStruct = Self;
     type Variant = Self;
 
     #[inline]
@@ -68,11 +71,27 @@ where
     }
 }
 
-impl<'de, T> PairDecoder<'de> for Never<T>
+impl<E> AsDecoder for Never<E>
 where
-    T: Decoder<'de>,
+    E: Error,
 {
-    type Error = T::Error;
+    type Error = E;
+
+    type Decoder<'this> = Never<E>
+    where
+        Self: 'this;
+
+    #[inline]
+    fn as_decoder(&self) -> Result<Self::Decoder<'_>, Self::Error> {
+        match self._never {}
+    }
+}
+
+impl<'de, E> PairDecoder<'de> for Never<E>
+where
+    E: Error,
+{
+    type Error = E;
 
     type First<'this> = Self
     where
@@ -96,11 +115,11 @@ where
     }
 }
 
-impl<'de, T> VariantDecoder<'de> for Never<T>
+impl<'de, E> VariantDecoder<'de> for Never<E>
 where
-    T: Decoder<'de>,
+    E: Error,
 {
-    type Error = T::Error;
+    type Error = E;
 
     type Tag<'this> = Self
     where
@@ -129,11 +148,11 @@ where
     }
 }
 
-impl<'de, T> PairsDecoder<'de> for Never<T>
+impl<'de, E> PairsDecoder<'de> for Never<E>
 where
-    T: Decoder<'de>,
+    E: Error,
 {
-    type Error = T::Error;
+    type Error = E;
 
     type Decoder<'this> = Self
     where
@@ -148,13 +167,18 @@ where
     fn next(&mut self) -> Result<Option<Self::Decoder<'_>>, Self::Error> {
         match self._never {}
     }
+
+    #[inline]
+    fn end(self) -> Result<(), Self::Error> {
+        match self._never {}
+    }
 }
 
-impl<'de, T> SequenceDecoder<'de> for Never<T>
+impl<'de, E> SequenceDecoder<'de> for Never<E>
 where
-    T: Decoder<'de>,
+    E: Error,
 {
-    type Error = T::Error;
+    type Error = E;
 
     type Decoder<'this> = Self
     where
@@ -169,13 +193,18 @@ where
     fn next(&mut self) -> Result<Option<Self::Decoder<'_>>, Self::Error> {
         match self._never {}
     }
+
+    #[inline]
+    fn end(self) -> Result<(), Self::Error> {
+        match self._never {}
+    }
 }
 
-impl<'de, T> PackDecoder<'de> for Never<T>
+impl<'de, E> PackDecoder<'de> for Never<E>
 where
-    T: Decoder<'de>,
+    E: Error,
 {
-    type Error = T::Error;
+    type Error = E;
 
     type Decoder<'this> = Self
     where
@@ -185,6 +214,11 @@ where
     fn next(&mut self) -> Result<Self::Decoder<'_>, Self::Error> {
         match self._never {}
     }
+
+    #[inline]
+    fn end(self) -> Result<(), Self::Error> {
+        match self._never {}
+    }
 }
 
 impl<T> Encoder for Never<T>
@@ -199,7 +233,6 @@ where
     type Tuple = Self;
     type Map = Self;
     type Struct = Self;
-    type TupleStruct = Self;
     type Variant = Self;
 
     #[inline]
@@ -257,7 +290,7 @@ where
     type First<'this> = Self
     where
         Self: 'this;
-    type Second = Self;
+    type Second<'this> = Self where Self: 'this;
 
     #[inline]
     fn first(&mut self) -> Result<Self::First<'_>, Self::Error> {
@@ -265,7 +298,12 @@ where
     }
 
     #[inline]
-    fn second(self) -> Result<Self::Second, Self::Error> {
+    fn second(&mut self) -> Result<Self::Second<'_>, Self::Error> {
+        match self._never {}
+    }
+
+    #[inline]
+    fn end(self) -> Result<Self::Ok, Self::Error> {
         match self._never {}
     }
 }
